@@ -1,4 +1,4 @@
-/* -*- mode: c; c-basic-offset: 4; -*- */
+/* -*- tab-width:8 c-basic-offset:4  indent-tabs-mode:nil -*- */
 #if HAVE_CONFIG_H
 #include "daq_config.h"
 #endif /* HAVE_CONFIG_H */
@@ -25,6 +25,7 @@
 #include "nds_logging.h"
 
 #define MAX_GROUP_LIST   4096
+#define MAX_EPOCH_LIST   65536UL
 #define MAX_CHANNEL_LIST 209715200
 
 #define NEW_VECT(type,dim) ((type*)malloc(dim*sizeof(type)))
@@ -37,6 +38,7 @@ enum pgm_mode {
     chan_data,
     server_protocol,
     server_version,
+    epoch_list,
     cmd_exec
 };
 
@@ -138,7 +140,7 @@ dump_data(daq_t* daq, size_t spl, const char* fmt) {
 	    break;
 	}
 	default:
-	    printf("Channel: %s data of unknown/unsupprted type (%i)\n", 
+	    printf("Channel: %s data of unknown/unsupported type (%i)\n", 
 		   chan->name, chan->data_type);
 	}
     }
@@ -230,7 +232,14 @@ main(int argc, const char* argv[]) {
 	else if (!strcmp(argv[iarg], "--epoch")) {
 	    if (++iarg == argc) syntax = 1;
 	    else                epoch = argv[iarg];
+	}
+
+	/*------------------------------  Print list of epochs       */
+	else if (!strcmp(argv[iarg], "-E") || 
+		 !strcmp(argv[iarg], "--epoch-list")) {
+	    mode = epoch_list;
 	} 
+
 	else if (!strcmp(argv[iarg], "--channel-list-pre-fetch")) {
 	    optimize_channel_fetch = 0;
 	} 
@@ -332,20 +341,35 @@ main(int argc, const char* argv[]) {
 	fprintf(stderr, "No channels requested!\n");
 	syntax = 1;
     }
+    else if ((mode == chan_crc || mode == chan_list || mode== chan_count) && 
+             iarg < argc-1) {
+	fprintf(stderr, "Channel lists accept only a single name pattern\n");
+	syntax = 1;
+    }
     if (syntax) {
-	fprintf(stderr, "Command syntax: \n");
-	fprintf(stderr, "nds_query [--version] [--server-version] [--server-protocol]\n");
-	fprintf(stderr, "          [--channel-list-pre-fetch] [--count | -k] [--hash | -h]\n");
-	fprintf(stderr, "          [-g] [-l] [-n <server>] [-p <port>] [--epoch <epik>]\n");
+	fprintf(stderr, "Data query syntax: \n");
+	fprintf(stderr, "nds_query [-n <server>] [-p <port>] [-12v] [--epoch <epik>]\n");
 	fprintf(stderr, "          [-s <start-gps>] [-e <end-gps>] [-d <delta-t>]\n");
-	fprintf(stderr, "          [<channel-list>]\n");
-	fprintf(stderr, "where: \n");
-	fprintf(stderr, "    -d      Time stride [%i]\n", DEFAULT_DELTA);
-	fprintf(stderr, "    -e      End gps time [start_gps + delta]\n");
+        fprintf(stderr, "          [-t <chan-type>] [--channel-list-pre-fetch] <channel-list>\n\n");
+
+        fprintf(stderr, "Channel List query syntax: \n");
+        fprintf(stderr, "nds_query [-n <server>] [-p <port>] [-12] [--epoch <epik>]\n");
+        fprintf(stderr, "          [-s <start-gps>] [-t <chan-type>]\n");
+        fprintf(stderr, "          { {--count | -k} | {--hash | -h} | -l }\n");
+        fprintf(stderr, "          [<channel-pattern>]\n\n");
+        fprintf(stderr, "Status list query syntax: \n");
+        fprintf(stderr, "nds_query [-n <server>] [-p <port>] [-12]\n");
+        fprintf(stderr, "          [--version] [--server-version] [--server-protocol]\n");
+        fprintf(stderr, "          [--epoch-list | -E] [-g]\n\n");
+        fprintf(stderr, "where: \n");
+        fprintf(stderr, "    -E      Request a list of epochs (NDS2 only)\n");
 	fprintf(stderr, "    -g      Request a list of channel groups (NDS1 only)\n");
 	fprintf(stderr, "    -h      Get hash of selected channels\n");
 	fprintf(stderr, "    -k      Get number of selected channels\n");
 	fprintf(stderr, "    -l      List available channels\n");
+
+	fprintf(stderr, "    -d      Time stride [%i]\n", DEFAULT_DELTA);
+	fprintf(stderr, "    -e      End gps time [start_gps + delta]\n");
 	fprintf(stderr, "    -n      Specify server IP address\n");
 	fprintf(stderr, "    -p      Specify port number [%i]\n", DAQD_PORT);
 	fprintf(stderr, "    -s      Specify start time (as GPS)\n");
@@ -364,6 +388,7 @@ main(int argc, const char* argv[]) {
 	fprintf(stderr, "    --server-version   print version of server and exit\n");
 	fprintf(stderr, "    --server-protocol  print protocol used by the server and exit\n");
 	fprintf(stderr, "    <channel-list> Space delimited channel list\n");
+ 	fprintf(stderr, "    <channel-pattern> bash-like regular expression\n");
 	fprintf(stderr, "If the protocol is not specified, nds_query tries\n");
 	fprintf(stderr, "nds2 protocol, and if this fails, reverts to nds1\n");
 	fprintf(stderr, "protocol\n");
@@ -479,7 +504,12 @@ main(int argc, const char* argv[]) {
 				__LINE__,
 				__FILE__ );
 	}
-	rc = daq_recv_channel_hash(&daq, &hash, &len, gps_start, c_type);
+ 	if (iarg < argc) {
+ 	    rc = daq_recv_hash_by_pattern(&daq, &hash, &len, gps_start, c_type,
+                                          argv[iarg]);
+ 	} else {
+            rc = daq_recv_channel_hash(&daq, &hash, &len, gps_start, c_type);
+	}
 	if (rc)
 	    printf("Error reading channel hash: %s\n", daq_strerror(rc));
 	else
@@ -487,6 +517,41 @@ main(int argc, const char* argv[]) {
 	break;
     }
 
+    /*--------------------------------------------------------------------*
+     *                                                                    *
+     *      Get the channel list hash                                     *
+     *                                                                    *
+     *--------------------------------------------------------------------*/
+    case epoch_list: {
+	int len;
+	char* buf = malloc(MAX_EPOCH_LIST);
+
+	if ( nds_logging_check( NDS_LOG_GROUP_USER, 10 ) ) {
+	    nds_logging_printf( "DEBUG: %06d - %s\n",
+				__LINE__,
+				__FILE__ );
+	}
+	rc = daq_recv_epoch_list(&daq, buf, MAX_EPOCH_LIST, &len);
+	buf[ MAX_EPOCH_LIST - 1 ] = 0;
+	if (rc) {
+	    printf("Error reading epoch list: %s\n", daq_strerror(rc));
+	}
+	else {
+	    int ich, last=0;
+	    printf("NDS2 server epoch list:\n");
+	    for (ich=0; ich<len; ich++) {
+		if (buf[ich] == ' ') {
+		    buf[ich] = 0;
+		    if (ich != last) printf("  %s\n", buf+last);
+		    last = ich+1;
+		}
+	    }
+	    if (last < len) printf("  %s\n", buf+last);
+	}
+	free(buf);
+	break;
+    }
+
     /*--------------------------------------------------------------------*
      *                                                                    *
      *      Get the channel list                                          *
@@ -518,10 +583,18 @@ main(int argc, const char* argv[]) {
 	     */
 	} else {
 	    /*
-	     * Retrieve a list of channels by first retrieving the number
-	     * of channels
+	     *  Retrieve a list of channels by first retrieving the number
+	     *  of channels. Note that the full channel list is fetched unless
+             *  there is a single channel (pattern?) specified and the mode is
+             *  either non-data or optimized channel fetch is disabled.
 	     */
-	    rc = daq_recv_channel_list(&daq, 0, 0, &nAlloc, gps_start, c_type);
+ 	    if (iarg+1==argc && (optimize_channel_fetch || mode != chan_data)) {
+                rc = daq_recv_channels_by_pattern(&daq, 0, 0, &nAlloc, 
+                                                  gps_start, c_type,argv[iarg]);
+ 	    } else {
+ 		rc = daq_recv_channel_list(&daq, 0, 0, &nAlloc, gps_start, 
+ 					   c_type);
+ 	    }
 	    if (rc) {
 		printf("Error reading channel count: %s\n", daq_strerror(rc));
 		break;
@@ -546,8 +619,14 @@ main(int argc, const char* argv[]) {
 	    channel_list = NEW_VECT(daq_channel_t, (size_t)(nAlloc));
 	    
 	    /*-----------------------------  Read in the channel list        */
-	    rc = daq_recv_channel_list(&daq, channel_list, nAlloc, 
-				       &nChans, gps_start, c_type);
+ 	    if (iarg+1==argc && (optimize_channel_fetch || mode != chan_data)) {
+ 		rc = daq_recv_channels_by_pattern(&daq, channel_list, nAlloc, 
+						  &nChans, gps_start, c_type, 
+						  argv[iarg]);
+ 	    } else {
+ 		rc = daq_recv_channel_list(&daq, channel_list, nAlloc, &nChans,
+ 					   gps_start, c_type);
+ 	    }
 	    if (rc) {
 		printf("Error reading channel list: %s\n", daq_strerror(rc));
 		free(channel_list);
