@@ -75,7 +75,8 @@ enum nds2_errnum {
 	NDS2_UNEXPECTED_CHANNELS_RECEIVED, /* server sent more channels than expected */
 	NDS2_MINUTE_TRENDS, /* some minute trends were requested, but start and stop times were not divisible by 60 */
 	NDS2_TRANSFER_BUSY, /* another transfer is already in progress */
-    NDS2_NO_SUCH_CHANNEL, /* no such channel */
+	NDS2_NO_SUCH_CHANNEL, /* no such channel */
+	NDS2_ALREADY_CLOSED, /* connection already closed */
 	NDS2_ELAST /* last error code */
 };
 
@@ -210,6 +211,17 @@ DLL_EXPORT
 char *nds2_channel_type_to_string(nds2_channel_type);
 
 
+/**
+ * Allocate a new channel data structure. Caller is responsible for reclaiming
+ * memory with free().
+ */
+DLL_EXPORT
+nds2_channel *nds2_channel_alloc(
+	const char *name, int channel_type, int data_type, double sample_rate,
+	float signal_gain, float signal_slope, float signal_offset,
+	const char *signal_units);
+
+
 /**
  * Open a new NDS connection.
  */
@@ -222,7 +234,14 @@ nds2_connection *nds2_open(
 
 
 /**
- * Close an NDS connection.
+ * Close an NDS connection but do not free the connection data structure.
+ */
+DLL_EXPORT
+void nds2_shutdown(nds2_connection *connection);
+
+
+/**
+ * Close an NDS connection and free the connection data structure.
  */
 DLL_EXPORT
 void nds2_close(nds2_connection *connection);
